"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigStartup = void 0;
const common_1 = require("@nestjs/common");
const path_1 = __importDefault(require("path"));
const config_service_1 = require("./config.service");
const logger_service_1 = require("../logger/logger.service");
const directoryProvider_1 = require("../helpers/directoryProvider");
const fileProvider_1 = require("../helpers/fileProvider");
let ConfigStartup = class ConfigStartup {
    constructor(_logger, _configService, _directoryProvider, _fileProvider) {
        this._logger = _logger;
        this._configService = _configService;
        this._directoryProvider = _directoryProvider;
        this._fileProvider = _fileProvider;
    }
    async onApplicationShutdown(signal) {
        if (this._configService.isDevelopment) {
            return;
        }
        const msg = `Application is shutting down. Signal: ${signal}`;
        this._logger.warn(msg);
    }
    async onApplicationBootstrap() {
        await this.setupSymboldPaths();
    }
    async setupSymboldPaths() {
        if (process.env.NODE_ENV !== 'production') {
            this._logger.warn(`Symbold doesn't work in production mode!` +
                `If you're using symbold in production please change process.env flag`);
        }
        this._logger.debug(`Starting symbold path setup`);
        if (!(await this._fileProvider.exists(this._configService.path.destination))) {
            this._logger.warn(`Destination path doesn't exists. Symbold will create dest path.`);
        }
        if (!(await this._fileProvider.exists(this._configService.path.queue))) {
            this._logger.warn(`Queue path doesn't exists. Symbold will create queue path.`);
        }
        await this._directoryProvider.remove(this._configService.path.downloadPath);
        if (!this._configService.path.eventStoragePath) {
            this._configService.path.eventStoragePath = path_1.default.join(path_1.default.dirname(this._configService.path.eventPaths[0]), 'cur');
            this._logger.warn(`Event storage path doesn't exists. Using default: ${this._configService.path.eventStoragePath}`);
        }
        const requiredDirectories = new Set([
            this._configService.path.destination,
            ...this._configService.path.eventPaths,
            this._configService.path.downloadPath,
            this._configService.path.eventStoragePath,
        ]);
        if (this._configService.path.archivePath)
            requiredDirectories.add(this._configService.path.archivePath);
        if (!this._configService.path.createDir) {
            this._logger.log(`Create dir option is disabled. Skipping creating directory`);
            return;
        }
        for (const requiredDir of requiredDirectories) {
            if (await this._fileProvider.exists(requiredDir)) {
                this._logger.debug(`Directory ${requiredDir} already exists. Skipping...`);
                continue;
            }
            try {
                await this._directoryProvider.createDirRecursively(requiredDir);
            }
            catch (err) {
                this._logger.error(`Cannot create directory: ${requiredDir} Reason: ${err.message}`);
                throw err;
            }
        }
    }
};
ConfigStartup = __decorate([
    common_1.Injectable(),
    __metadata("design:paramtypes", [logger_service_1.LoggerService,
        config_service_1.ConfigService,
        directoryProvider_1.DirectoryProvider,
        fileProvider_1.FileProvider])
], ConfigStartup);
exports.ConfigStartup = ConfigStartup;
//# sourceMappingURL=config.startup.js.map