"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SymbolServerStartup = void 0;
const common_1 = require("@nestjs/common");
const fs = __importStar(require("fs"));
const path_1 = __importDefault(require("path"));
const readline_1 = __importDefault(require("readline"));
const serverCredentials_1 = require("../../database/entities/serverCredentials");
const credentialsType_1 = require("../../database/entities/types/credentialsType");
const config_service_1 = require("../config/config.service");
const logger_service_1 = require("../logger/logger.service");
const symbol_server_service_1 = require("./symbol-server.service");
let SymbolServerStartup = class SymbolServerStartup {
    constructor(_symbolServerService, _logger, _config) {
        this._symbolServerService = _symbolServerService;
        this._logger = _logger;
        this._config = _config;
        this._assetsDirectoryName = 'assets';
    }
    async onApplicationBootstrap() {
        if (!this._config.db.initialMigration) {
            return;
        }
        await this.recreateGlobalSymbolServers();
        await this.removeNotUsedSymbolServers();
    }
    async removeNotUsedSymbolServers() {
        const symbolServers = await this._symbolServerService.getGlobalSymbolServers();
        for (const symbolServer of symbolServers) {
            const exists = this._config.symbolServer.servers.some((n) => {
                var _a, _b;
                return n.url === symbolServer.url &&
                    (n.bucketName === ((_a = symbolServer.serverCredentials) === null || _a === void 0 ? void 0 : _a.bucketName) ||
                        n.aws_s3.bucket === ((_b = symbolServer.serverCredentials) === null || _b === void 0 ? void 0 : _b.bucketName));
            });
            if (!exists) {
                this._logger.warn(`Symbol server ${symbolServer.url} isn't anymore in symbold configuration file. Deleting....`);
                await this._symbolServerService.remove(symbolServer.id, true);
            }
        }
    }
    async recreateGlobalSymbolServers() {
        var _a;
        for (const symbolServer of this._config.symbolServer.servers) {
            const symbolServerEntity = await this._symbolServerService.getByUrl(symbolServer.url, ((_a = symbolServer === null || symbolServer === void 0 ? void 0 : symbolServer.aws_s3) === null || _a === void 0 ? void 0 : _a.bucket) || symbolServer.bucketName);
            if (symbolServerEntity) {
                this._logger.debug(`Skipping symbol server with url: ${symbolServer.url}. Symbol server is already in database`);
                continue;
            }
            this._logger.debug(`Adding new symbol server to database with url: ${symbolServer.url} ${symbolServer.name ? `and name ${symbolServer.name}` : ''}`);
            const useWhiteList = !!symbolServer.whitelist;
            const newSymbolServer = await this._symbolServerService.add({
                url: symbolServer.url,
                name: symbolServer.name,
                timeout: this._config.symbolServer.timeout * 1000 || 10000,
                queryBuilder: symbolServer.queryBuilder,
                whiteList: useWhiteList,
                hidden: symbolServer.hidden,
                retain: symbolServer.retain,
                serverCredentials: this.readServerCredentials(symbolServer),
            });
            if (useWhiteList) {
                if (symbolServer.local === true) {
                    symbolServer.whitelist = path_1.default.join(process.cwd(), this._assetsDirectoryName, symbolServer.whitelist);
                }
                if (fs.existsSync(symbolServer.whitelist)) {
                    await this.addWhiteListToSymbolServer(newSymbolServer.id, symbolServer.whitelist);
                }
            }
        }
    }
    readServerCredentials(symbolServer) {
        var _a, _b, _c, _d, _e, _f;
        if (symbolServer.username && symbolServer.password && !symbolServer.bucketName) {
            return {
                userName: symbolServer.username,
                password: symbolServer.password,
            };
        }
        if (!symbolServer.aws_s3 && !symbolServer.bucketName && !symbolServer.secretFile) {
            return undefined;
        }
        return {
            userName: (_a = symbolServer.aws_s3) === null || _a === void 0 ? void 0 : _a.aws_access_key_id,
            password: (_b = symbolServer.aws_s3) === null || _b === void 0 ? void 0 : _b.aws_secret_access_key,
            bucketName: ((_c = symbolServer.aws_s3) === null || _c === void 0 ? void 0 : _c.bucket) || symbolServer.bucketName,
            lowerFile: ((_d = symbolServer.aws_s3) === null || _d === void 0 ? void 0 : _d.lowerFile) || serverCredentials_1.SymbolNameType.NoChange,
            usePdb: ((_e = symbolServer.aws_s3) === null || _e === void 0 ? void 0 : _e.use_pdb) || true,
            lowerId: ((_f = symbolServer.aws_s3) === null || _f === void 0 ? void 0 : _f.lowerId) || serverCredentials_1.SymbolNameType.NoChange,
            secretFile: symbolServer.secretFile,
            type: symbolServer.secretFile ? credentialsType_1.CredentialsType.GCP : credentialsType_1.CredentialsType.AWS,
        };
    }
    async addWhiteListToSymbolServer(id, whiteListPath) {
        const rl = readline_1.default.createInterface({
            input: fs.createReadStream(whiteListPath),
            crlfDelay: Infinity,
        });
        return new Promise((res, rej) => {
            const whitelist = [];
            rl.on('line', (line) => {
                whitelist.push(line);
            });
            rl.on('close', async () => {
                await this._symbolServerService.addWhiteList(id, whitelist);
                res();
            });
        });
    }
};
SymbolServerStartup = __decorate([
    common_1.Injectable(),
    __metadata("design:paramtypes", [symbol_server_service_1.SymbolServerService,
        logger_service_1.LoggerService,
        config_service_1.ConfigService])
], SymbolServerStartup);
exports.SymbolServerStartup = SymbolServerStartup;
//# sourceMappingURL=symbolServer.startup.js.map